#include <TApplication.h>
#include <TCanvas.h>
#include <TROOT.h>

#include "Garfield/ComponentAnalyticField.hh"
#include "Garfield/DriftLineRKF.hh"
#include "Garfield/MediumMagboltz.hh"
#include "Garfield/Sensor.hh"
#include "Garfield/TrackSrim.hh"
#include "Garfield/ViewCell.hh"
#include "Garfield/ViewDrift.hh"
#include "Garfield/ViewSignal.hh"

#include <cstdlib>
#include <fstream>
#include <iostream>

using namespace Garfield;

int main(int argc, char* argv[]) {
    TApplication app("app", &argc, argv);

    MediumMagboltz gas;
    gas.LoadGasFile("helium3_625_ar_375_64atm_3000K.gas");
    gas.LoadIonMobility("IonMobility_Ar+_Ar.txt");

    ComponentAnalyticField cmp;
    cmp.SetMedium(&gas);

    const double rWire = 1e-3;
    const double rTube = 0.74;
    const double vWire = 2700.;
    const double vTube = 0.;
    cmp.AddWire(0., 0., 2 * rWire, vWire, "s", 30.);
    cmp.AddTube(rTube, vTube, 0, "t");
    cmp.AddReadout("s");

    const double tStep = 1;
    Sensor sensor;
    sensor.AddComponent(&cmp);
    sensor.AddElectrode(&cmp, "s");
    sensor.SetTimeWindow(-tStep / 2, tStep, 2000000);
    sensor.ClearSignal();

    TrackSrim track;
    track.SetSensor(&sensor);
    track.ReadFile("proton_in_he3-argon(gas).txt");
    track.SetKineticEnergy(0.65e6);

    DriftLineRKF drift;
    drift.SetSensor(&sensor);
    drift.SetMaximumStepSize(5.e-4);
    drift.EnableIonTail();
    drift.EnableSignalCalculation();

    TCanvas* cD = new TCanvas("cD", "", 600, 600);
    ViewCell cellView;
    ViewDrift driftView;
    cellView.SetCanvas(cD);
    cellView.SetComponent(&cmp);
    driftView.SetCanvas(cD);
    drift.EnablePlotting(&driftView);
    track.EnablePlotting(&driftView);

    int nce;
    double xc, yc, zc, tc, eDepc, extra;

    const double rTrack = 0.3;
    track.NewTrack(rTrack, rTrack, 0, 0, -1, 1, 0);
    while (track.GetCluster(xc, yc, zc, tc, nce, eDepc, extra)) {
        drift.SetIonSignalScalingFactor(nce);
        drift.DriftIon(xc, yc, zc, tc);
        drift.SetElectronSignalScalingFactor(nce);
        drift.DriftElectron(xc, yc, zc, tc);
    }

    cD->Clear();
    cellView.Plot2d();
    driftView.Plot(true, false);

    ViewSignal signalView;
    sensor.IntegrateSignals();
    signalView.SetSensor(&sensor);
    signalView.PlotSignal("s", "tei");

    app.Run(kTRUE);
    return 0;
}